#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/uio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "../packet.h"
#include "../htonl.h"

#define MAX_MESG_SIZE 2048
unsigned char mesg[MAX_MESG_SIZE];
PacketStruct *packet = (PacketStruct *)mesg;

FILE *logfile = NULL;
const char *logbase = ".";
static time_t logtime = 0;

static void
open_log (time_t when)
{
  char tmp[1000];
  struct tm *tm;

  if ((when - logtime) < 60)
    return;
  if (logfile)
    fclose (logfile);

  tm = localtime (&when);
  tm->tm_mon ++;
  tm->tm_year += 1900;

  sprintf(tmp, "%s/%04d/%02d/%02d", logbase, tm->tm_year, tm->tm_mon, tm->tm_mday);
  logfile = fopen(tmp, "a");
  if (!logfile)
    {
      sprintf(tmp, "%s/%04d", logbase, tm->tm_year);
      mkdir(tmp, 0777);
      sprintf(tmp, "%s/%04d/%02d", logbase, tm->tm_year, tm->tm_mon);
      mkdir(tmp, 0777);
      sprintf(tmp, "%s/%04d/%02d/%02d", logbase, tm->tm_year, tm->tm_mon, tm->tm_mday);
      logfile = fopen(tmp, "a");
      if (!logfile)
	{
	  perror(tmp);
	  exit(1);
	}
    }

  logtime = when - when%60;
}

main(int argc, char **argv)
{
  int udp_socket;
  struct sockaddr_in serverName;
  struct sockaddr_in clientName;
  int client_length, size;
  unsigned char *uc;
  int i;
  time_t now;

  if (argc > 1)
    logbase = argv[1];

  while (1)
    {
      udp_socket = socket (PF_INET, SOCK_DGRAM, IPPROTO_UDP);
      if (udp_socket == -1)
	{
	  perror("socket");
	  sleep(1);
	  continue;
	}

      memset(&serverName, 0, sizeof(serverName));
      memset(&clientName, 0, sizeof(clientName));

      serverName.sin_family = AF_INET;
      serverName.sin_addr.s_addr = htonl(INADDR_ANY);
      serverName.sin_port = htons(PM_PORT);

      if (bind(udp_socket, (struct sockaddr *)&serverName, sizeof(serverName)) < 0)
	{
	  perror("bind()");
	  sleep(1);
	  continue;
	}

      while (1)
	{
	  size = recvfrom(udp_socket, mesg,
			  MAX_MESG_SIZE, 0,
			  (struct sockaddr *) &clientName,
			  &client_length);
	  if (size == -1)
	    {
	      perror("recvfrom()");
	      break;
	    }

	  if (memcmp (packet->magic, "POWERMTR", 8) != 0)
	    continue;
	  if (htonl(packet->id) < 1 || htonl(packet->id) > 4)
	    continue;
  
	  time(&now);
	  open_log (now);

	  fprintf (logfile, "%lu %u %08x", now, htonl(packet->uptime), htonl(packet->id));
	       
	  for (i=0; i<16; i++)
	    {
	      long long e = htonll(packet->aenergy[i]);
	      long p = htonl(packet->laenergy[i]);

	      //fprintf(logfile, " %lld %ld", e, p);
	      fprintf(logfile, " %lld", e);
	    }
	  fprintf (logfile, "\n");
	  fflush (logfile);
	}

      close(udp_socket);
    }
}
